#include "StdAfx.h"
#include "IntervalImpl.h"
#include "Signature.h"
#include <strstream>
#include <boost/algorithm/string/split.hpp>
#include <boost/algorithm/string/predicate.hpp>

using namespace Eyw;

////////////////////////////////////////////////////////////////////////////////////
// Signature
datatype_class_registrant g_Interval( 
	datatype_class_registrant::datatype_id( "Interval" )
		.begin_language( EYW_LANGUAGE_US_ENGLISH )
			.name( "Interval" )
			.description( "An interval in the floating/point domain" )
		.end_language()	
		.begin_authors()
			.author( EYW_EYWWEEK2010_CATALOG_AUTHOR_ID )
		.end_authors()
		.begin_licences()
			.licence( EYW_EYWWEEK2010_LICENSE_ID )
		.end_licences()
		.begin_companies()
			.company( EYW_EYWWEEK2010_COMPANY_ID )
		.end_companies()
		.default_factory<CIntervalImpl>()
		.default_init_info_factory<CIntervalInitInfoImpl>()
	);

////////////////////////////////////////////////////////////////////////////////////
CIntervalInitInfoImpl::CIntervalInitInfoImpl( const Eyw::OBJECT_CREATIONCTX* pCtx )
:	Eyw::CDatatypeInitInfoImpl<IIntervalInitInfo>( pCtx )
{
}

CIntervalInitInfoImpl::~CIntervalInitInfoImpl()
{
}

////////////////////////////////////////////////////////////////////////////////////
// Factory
Eyw::IObject* CIntervalImpl::CreateInstance( const Eyw::OBJECT_CREATIONCTX* pCtx )
{
	CIntervalImpl* pDatatype = new CIntervalImpl( pCtx );
	if( pDatatype == 0)
		throw Eyw::CMemoryExceptionImpl();
	return pDatatype;
}

////////////////////////////////////////////////////////////////////////////////////
CIntervalImpl::CIntervalImpl( const Eyw::OBJECT_CREATIONCTX* pCtx )
:	Eyw::CDatatypeImpl<IInterval>( pCtx ),
	_min( 0.0 ),
	_max( 1.0 )
{
}

EYWAPI CIntervalImpl::~CIntervalImpl()
{
}

Eyw::PMEM_BUF EYWAPI CIntervalImpl::Serialize() const
{
	Eyw::PMEM_BUF pRet = NULL;
	pRet = Eyw::CreateMemBuffer( sizeof(double)*2 );
	if( pRet != 0 )
	{
		double* pTarget=(double*)pRet->_pBuf;
		pTarget[0]=_min;
		pTarget[1]=_max;
	}
	return pRet;
}

void EYWAPI CIntervalImpl::ReleaseBuffer( Eyw::PMEM_BUF pBuf ) const
{
	DestroyMemBuffer( pBuf );
}

bool EYWAPI CIntervalImpl::Unserialize( Eyw::PCMEM_BUF pBuf )
{
	if( pBuf == 0 )
		return true;

	if(pBuf->_mSize != (2*sizeof(double)) )
		return false;

	double* pSource = (double*)pBuf->_pBuf;
	_min = pSource[0];
	_max = pSource[1];

	return true;
}

Eyw::STR EYWAPI CIntervalImpl::SerializeAsText() const
{
	std::string s =
		boost::lexical_cast<std::string>( _min ) +
		"," +
		boost::lexical_cast<std::string>( _max );
	return _strdup( s.c_str() );
}

bool EYWAPI CIntervalImpl::UnserializeFromText(Eyw::CSTR str)
{
	std::vector<std::string> tokens;
	boost::split( tokens, str, boost::is_any_of( ",") );
	if( tokens.size() != 2 )
		return false;
	try
	{
		_min = boost::lexical_cast<double>( tokens[ 0 ] );
		_max = boost::lexical_cast<double>( tokens[ 1 ] );
		return true;
	}
	catch( const std::exception& )
	{
		return false;
	}
}

void EYWAPI CIntervalImpl::ReleaseText( Eyw::STR str ) const
{
	if( str )
		free( str );
}

void EYWAPI CIntervalImpl::CopyFrom( const ISerialize* pSourceDatatype )
{
	const CIntervalImpl* sourcePtr = Cast<const CIntervalImpl*>(pSourceDatatype);
	_min = sourcePtr->_min;
	_max = sourcePtr->_max;
}

void CIntervalImpl::SetMin( double val )
{
	_min = val;
}

void CIntervalImpl::SetMax( double val )
{
	_max = val;
}

double CIntervalImpl::GetMin() const
{
	return _min;
}

double CIntervalImpl::GetMax() const
{
	return _max;
}

////////////////////////////////////////////////////////////////////////////////////
