#include "StdAfx.h"
#include "Vector.h"
#include "Signature.h"
#include "resource.h"

using namespace Eyw;

////////////////////////////////////////////////////////////////////////////////////
// Signature
block_class_registrant g_Vector( 
	block_class_registrant::block_id( "Vector" )
		.begin_language( EYW_LANGUAGE_US_ENGLISH )
			.name( "Vector" )
			.description( "A block which stores an array of datatypes"	)
			.libraries( "DataStructures" )
			.bitmap( IDB_VECTOR_BITMAP )
		.end_language()	
		.begin_authors()
			.author( EYW_EYWWEEK2010_CATALOG_AUTHOR_ID )
		.end_authors()
		.begin_companies()
			.company( EYW_EYWWEEK2010_COMPANY_ID )
		.end_companies()
		.begin_licences()
			.licence( EYW_EYWWEEK2010_LICENSE_ID )
		.end_licences()
		.default_factory< CVector >()
	);

//////////////////////////////////////////////////////////////////////
// Identifiers
#define PARAMETER_SIZE "size"
#define PARAMETER_RESET "reset"
#define PARAMETER_ENABLE_READ "enable_read"
#define PARAMETER_READ_POSITION "read_position"
#define PARAMETER_ENABLE_WRITE "enable_write"
#define PARAMETER_WRITE_POSITION "write_position"
#define INPUT_INPUT "input"
#define OUTPUT_OUTPUT "output"


/////////////////////////////////
CVector::CVector( const Eyw::OBJECT_CREATIONCTX* ctxPtr )
:	Eyw::CBlockImpl( ctxPtr )
{
	_pInInput=NULL;
	_pOutOutput=NULL;

	_schedulingInfoPtr->SetActivationEventBased( true );
	_schedulingInfoPtr->GetEventBasedActivationInfo()->SetActivationOnInputChanged( INPUT_INPUT, true );

}

CVector::~CVector()
{
}

void CVector::InitSignature()
{
	_pParamSize= Eyw::Cast<Eyw::IInt*>(
	                     SetParameter(Eyw::pin::id(PARAMETER_SIZE)
	                         .name("size")
	                         .description("The size of the internal array")
							 .is_run_time(false)
							 .set_int_domain()
								.min(1)
	                         )->GetDatatype() );
	_pParamSize->SetValue( 1 );
	_pParamReset= Eyw::Cast<Eyw::ITrigger*>(
	                     SetParameter(Eyw::pin::id(PARAMETER_RESET)
	                         .name("reset")
	                         .description("A command to clear all internal datatypes")
	                         .type<Eyw::ITrigger>()
	                         )->GetDatatype() );
	_pParamEnable_read= Eyw::Cast<Eyw::IBool*>(
	                     SetParameter(Eyw::pin::id(PARAMETER_ENABLE_READ)
	                         .name("enable_read")
	                         .description("Specifies whether reading (i.e., copying to output) is enabled")
	                         .type<Eyw::IBool>()
	                         )->GetDatatype() );
	_pParamEnable_read->SetValue(true);
	_pParamRead_position= Eyw::Cast<Eyw::IInt*>(
	                     SetParameter(Eyw::pin::id(PARAMETER_READ_POSITION)
	                         .name("read_position")
	                         .description("The position in the internal array of the datatype copied to the output pin")
	                         .type<Eyw::IInt>()
	                         )->GetDatatype() );
	_pParamEnable_write= Eyw::Cast<Eyw::IBool*>(
	                     SetParameter(Eyw::pin::id(PARAMETER_ENABLE_WRITE)
	                         .name("enable_write")
	                         .description("Specifies whether writing is enabled")
	                         .type<Eyw::IBool>()
	                         )->GetDatatype() );
	_pParamEnable_write->SetValue(true);
	_pParamWrite_position= Eyw::Cast<Eyw::IInt*>(
	                     SetParameter(Eyw::pin::id(PARAMETER_WRITE_POSITION)
	                         .name("write_position")
	                         .description("Position in the internal array where the input datatype will be copied")
	                         .type<Eyw::IInt>()
	                         )->GetDatatype() );
	SetInput(Eyw::pin::id(INPUT_INPUT)
	    .name("input")
	    .description("Input")
	    .type<Eyw::IDatatype>()
		.set_required_interfaces()
			.requires<IScalarArithOpDouble>()
	    );
	SetOutput(Eyw::pin::id(OUTPUT_OUTPUT)
	    .name("output")
	    .description("Output")
	    .type<Eyw::IDatatype>()
		.inherited_id(INPUT_INPUT)
	    );

}

void CVector::CheckSignature()
{
	_pParamSize=get_parameter_datatype<Eyw::IInt>(PARAMETER_SIZE);
	_pParamReset=get_parameter_datatype<Eyw::ITrigger>(PARAMETER_RESET);
	_pParamEnable_read=get_parameter_datatype<Eyw::IBool>(PARAMETER_ENABLE_READ);
	_pParamRead_position=get_parameter_datatype<Eyw::IInt>(PARAMETER_READ_POSITION);
	_pParamEnable_write=get_parameter_datatype<Eyw::IBool>(PARAMETER_ENABLE_WRITE);
	_pParamWrite_position=get_parameter_datatype<Eyw::IInt>(PARAMETER_WRITE_POSITION);
	_signaturePtr->GetInputs()->FindItem( INPUT_INPUT );
	_signaturePtr->GetOutputs()->FindItem( OUTPUT_OUTPUT );

}

void CVector::DoneSignature()
{
	_pParamSize=NULL;
	_pParamReset=NULL;
	_pParamEnable_read=NULL;
	_pParamRead_position=NULL;
	_pParamEnable_write=NULL;
	_pParamWrite_position=NULL;

}

// Actions
bool CVector::Init() throw()
{
    try
    {

		_pInInput = get_input_datatype<Eyw::IDatatype>( INPUT_INPUT );
		_pOutOutput = get_output_datatype<Eyw::IDatatype>( OUTPUT_OUTPUT );

		_pOutOutput->InitInstance( _pInInput->GetInitInfo() );

		ResetInternalArray();

    	return true;
    }
    catch(...)
    {
        return false;
    }
}

bool CVector::Start() throw()
{
    try
    {
    	// TODO: add your logic
    	return true;
    }
    catch(...)
    {
        return false;
    }
}

bool CVector::Execute() throw()
{
    try
    {
    	if( _pParamEnable_write->GetValue() )
		{
			int write_position = _pParamWrite_position->GetValue();
			if( (write_position >= 0) && (write_position < _vector.size()) )
			{
				_vector[write_position]->CopyFrom( _pInInput.get() );
			}
		}

		if( _pParamEnable_read->GetValue() )
		{
			int read_position = _pParamRead_position->GetValue();
			if( (read_position >= 0) && (read_position < _vector.size()) )
			{
				_pOutOutput->CopyFrom( _vector[read_position].get() );
				_pOutOutput->SetCreationTime( _clockPtr->GetTime() );
			}
		}
    }
    catch(...)
    {
    }
	return true;
}

void CVector::Stop() throw()
{
    try
    {
    }
    catch(...)
    {
    }
}

void CVector::Done() throw()
{
    try
    {
		_pInInput = NULL;
		_pOutOutput = NULL;
		_vector.clear();

    }
    catch(...)
    {
    }
}

void CVector::OnChangedParameter( const std::string& parameterId )
{
	if( IsRunTime() && (parameterId == PARAMETER_RESET) )
	{
		ResetInternalArray();
	}

}

void CVector::ResetInternalArray()
{
	int size = _pParamSize->GetValue();
	_vector.resize( size );
	for( int i = 0; i < size; ++i )
	{
		_vector[ i ] = create_initialized_datatype_clone( _kernelServicesPtr, _pInInput );
	}
}
